<?php
namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\CustomizacaoVendaService;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\ParameterBag;

class CustomizacaoVenda extends CrudWithParameterBag {

    protected $dataSourceName = 'customizacaoVenda';
    protected $dataSourceName1 = 'relacao';
    protected $dataSourceName3 = 'relacaoUltatu';
    protected $dataSourceName2 = 'customizacaoField';
    /**
     * @var CustomizacaoVendaService
     */
    protected $CustomizacaoVendaService;
    protected $parameterBagColumns = array('NRORG');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, CustomizacaoVendaService $customizacaoVendaService) {
        $this->customizacaoVendaService = $customizacaoVendaService;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $dataSet = $request->getDataSet();
        foreach($dataSet->getRows() as &$row) {
            if(!isset($row['NRRELAATRIBUTOCONFIGURACAO'])){
                $row['NRRELAATRIBUTOCONFIGURACAO'] = $this->customizacaoVendaService->getNextId();
                $row['CDOPERINCLUSAO'] = $row['CDOPERADOR'];
                $row['NRORGINCLUSAO'] = $row['NRORG'];
                $row['CDOPERULTATU'] = $row['CDOPERADOR'];
                $row['NRORGULTATU'] = $row['NRORG'];
                $row['DTULTATU'] = date("d/m/Y H:i:s");
                $row['DTINCLUSAO'] = $row['DTULTATU'];
                $dataSet->setDataSourceName($this->dataSourceName1);
            }
            $row['DTULTATU'] = date("d/m/Y H:i:s");
            $row['DTINCLUSAO'] = $row['DTULTATU'];
            if(!$row['__is_new']) {
                $row['IDATIVO'] = 'S';
                $row['CDOPERULTATU'] = $row['CDOPERADOR'];
                $row['NRORGULTATU'] = $row['NRORG'];
                $row['DTULTATU'] = date("d/m/Y H:i:s");
                $dataSet->setDataSourceName($this->dataSourceName3);
            }
        }
        try{
            $this->dataSourceManager->persist($dataSet);
            $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));

        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));

        }
    }


    public function saveVenda (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $dataSet = $request->getDataSet();
            $dataSet->getRows()[count($dataSet->getRows())-1]['NRATRIBUTOVENDA'] = $this->customizacaoVendaService->getNextVendaId();
            $dataSet->setDataSourceName($this->dataSourceName);
            $this->dataSourceManager->persist($dataSet);
            $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $response->addDataSet($dataSet);
    }

    public function findCustomizacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $response->addDataSet($dataSet);
    }

    public function remove(DTO\Request\DataSet $request, DTO\Response $response){
        foreach ($request->getDataSet()->getRows() as $row) {
            $row['IDATIVO'] = 'N';
            $row['DTULTATU'] = date("d/m/Y H:i:s");
            $row['CDOPERULTATU'] = $row['CDOPERADOR'];
            $row['NRORGULTATU'] = $row['NRORG'];
            try {
                $this->dataSourceManager->persist(new DataSet($this->dataSourceName3, array($row)));
            } catch (Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
        $response->addNotification(new Notification("Dados excluídos com sucesso!", Notification::TYPE_SUCCESS));
    }

}