<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelCancelamentoItem {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function find($conditions) {
        $params = array(
         	":CDFILIAL" => $conditions['CDFILIAL'],
         	":TCDFILIAL" => $conditions['TCDFILIAL'],
         	":CDLOJA" => $conditions['CDLOJA'],
         	":TCDLOJA" => $conditions['TCDLOJA'],
         	":CDVENDEDOR" => $conditions['CDVENDEDOR'],
         	":TCDVENDEDOR" => $conditions['TCDVENDEDOR'],
            ":DTINI" => $conditions['DTINI'],
            ":DTFIN" => $conditions['DTFIN'],
         	":NRORG" => $conditions['NRORG'],
         	":IDPRODPRODUZC" => $conditions['IDPRODPRODUZC'],
         	":TIDPRODPRODUZC" => $conditions['TIDPRODPRODUZC']
 		); 
 		return $this->entityManager->getConnection()->fetchAll(self::GET_REL_CANC_ITEM, $params);
    }
    
    public function buildParamsFaturamento($conditions){
        
        if(!array_key_exists('CDFILIAL', $conditions)) {
            $conditions['CDFILIAL'] = '';
            $conditions['TCDFILIAL'] = 'T';
        } else {
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $conditions['TCDFILIAL'] = '';
        }
        if(!array_key_exists('CDLOJA', $conditions)) {
            $conditions['CDLOJA'] = '';
            $conditions['TCDLOJA'] = 'T';
        } else {
            $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
            $conditions['TCDLOJA'] = '';
        }
        if(!array_key_exists('GARCON', $conditions)) {
            $conditions['CDVENDEDOR'] = '';
            $conditions['TCDVENDEDOR'] = 'T';
        } else {
            $conditions['CDVENDEDOR'] = implode('_', $conditions['GARCON']);
            $conditions['TCDVENDEDOR'] = '';
        }
        if(array_key_exists('DTENTRVENDA', $conditions)) {
            $conditions['DTINI'] = $conditions['DTENTRVENDA'][0];
            $conditions['DTFIN'] = $conditions['DTENTRVENDA'][1];
            unset($conditions['DTENTRVENDA']);
        }
        if(array_key_exists('IDPRODPRODUZC',$conditions)){
            if(in_array('N',$conditions['IDPRODPRODUZC']) && !in_array('S',$conditions['IDPRODPRODUZC'])){
                $conditions['TIDPRODPRODUZC'] = '';
                $conditions['IDPRODPRODUZC'] = 'N';
            }else if(in_array('N',$conditions['IDPRODPRODUZC']) && in_array('S',$conditions['IDPRODPRODUZC'])){
                $conditions['TIDPRODPRODUZC'] = 'T';
                $conditions['IDPRODPRODUZC'] = '';
            }else{
                $conditions['TIDPRODPRODUZC'] = '';
                $conditions['IDPRODPRODUZC'] = 'S';
            }
            
        }else{
            $conditions['TIDPRODPRODUZC'] = 'T';
            $conditions['IDPRODPRODUZC'] = '';
        }
        return $conditions;
    }
    
    public function reorderResultsbyDate($result){
        $filial = 'CDFILIAL';
        $dataVenda = 'DATAVENDA';
        $currentFilial = $result[0][$filial]; 
        $data = $result[0][$dataVenda]; 
        
        $i = 0;
                
        foreach($result as $index => $res) {
            
            if($res[$filial] === $currentFilial) {
                if($data === $res[$dataVenda]) {
                    $result[$index][$dataVenda] = str_pad(($i + 1), 3, "0", STR_PAD_LEFT) . ' - ' . $result[$index][$dataVenda]; 
                } else {
                    $data = $result[$index][$dataVenda];
                    $i++;
                    $result[$index][$dataVenda] = str_pad(($i + 1), 3, "0", STR_PAD_LEFT) . ' - ' . $result[$index][$dataVenda]; 
                }
            } else {
                $currentFilial = $res[$filial];
                $i = -1;
                
                if($data === $res[$dataVenda]) {
                    $result[$index][$dataVenda] = str_pad(($i + 1), 3, "0", STR_PAD_LEFT) . ' - ' . $result[$index][$dataVenda]; 
                } else {
                    $data = $result[$index][$dataVenda];
                    $i++;
                    $result[$index][$dataVenda] = str_pad(($i + 1), 3, "0", STR_PAD_LEFT) . ' - ' . $result[$index][$dataVenda]; 
                }
            }
        }
        
        return $result;
    }
    
  const GET_REL_CANC_ITEM = "
    SELECT F.CDFILIAL || ' - ' || F.NMFILIAL || ' Loja:' || L.CDLOJA || ' - ' || L.NMLOJA AS LABELFILIAL,
      F.CDFILIAL || ' - ' || F.NMFILIAL AS LABELUNIDADE,
      'Todos' AS LABELLOJA,
      TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY') AS DATAVENDA,
      I.CDFILIAL,
      F.NMFILIAL,
      VE.CDVENDEDOR,
      VE.NMRAZSOCVEN,
      VE.CDVENDEDOR || ' - ' || VE.NMRAZSOCVEN AS GARCON,
      NVL(I.CDSUPERVISOR, V.CDSUPERVISOR) AS CDSUPERVISOR,
      O.NMOPERADOR,
      V.NRMESA,
      I.CDPRODUTO,
      P.CDARVPROD,
      P.NMPRODUTO,
      I.DSOBSCANITE,
      (I.QTPRODVENDC * (I.VRUNITVENDC + I.VRUNITVENCLC)) AS VRUNITVEND,
      V.IDORIGEMVENDA,
      CASE
        WHEN NVL(I.CDSUPERVISOR, V.CDSUPERVISOR) is NULL THEN 'Nao atribuido'
        ELSE NVL(I.CDSUPERVISOR, V.CDSUPERVISOR) || ' - ' || O.NMOPERADOR
      END SUPERVISOR,
      CASE
        I.IDPRODPRODUZC
        WHEN 'S' THEN 'Produzido'
        ELSE 'Não Produzido'
      END IDPRODPRODUZC
    FROM VENDA V
      JOIN ITVENDACAN I ON V.CDFILIAL = I.CDFILIAL
      AND V.NRSEQVENDA = I.NRSEQVENDA
      AND V.CDCAIXA = I.CDCAIXA
      AND V.NRORG = I.NRORG
      JOIN FILIAL F ON F.CDFILIAL = I.CDFILIAL
      AND F.NRORG = I.NRORG
      LEFT JOIN VENDEDOR VE ON V.CDVENDEDOR = VE.CDVENDEDOR
      AND V.NRORG = VE.NRORG
      JOIN PRODUTO P ON I.CDPRODUTO = P.CDPRODUTO
      AND I.NRORG = P.NRORG
      LEFT JOIN OPERADOR O ON O.CDOPERADOR = NVL(I.CDSUPERVISOR, V.CDSUPERVISOR)
      AND I.NRORG = O.NRORG
      JOIN LOJA L ON V.CDFILIAL = L.CDFILIAL
      AND V.CDLOJA = L.CDLOJA
      AND V.NRORG = L.NRORG
    WHERE (
        INSTR(:CDFILIAL, V.CDFILIAL) <> 0
        OR (:TCDFILIAL = 'T')
      )
      AND (
        (INSTR(:CDLOJA, V.CDFILIAL || V.CDLOJA) <> 0)
        OR (:TCDLOJA = 'T')
      )
      AND (
        (INSTR(:CDVENDEDOR, V.CDVENDEDOR) <> 0)
        OR (:TCDVENDEDOR = 'T')
      )
      AND V.DTENTRVENDA BETWEEN TO_DATE(:DTINI, 'DD/MM/YYYY') AND TO_DATE(:DTFIN, 'DD/MM/YYYY')
      AND (
        NVL(I.IDPRODPRODUZC, 'N') IN (:IDPRODPRODUZC)
        OR (:TIDPRODPRODUZC = 'T')
      )
      AND V.NRORG = :NRORG
    UNION ALL
    SELECT FI.CDFILIAL || ' - ' || FI.NMFILIAL || ' Loja:' || LO.CDLOJA || ' - ' || LO.NMLOJA AS LABELFILIAL,
      FI.CDFILIAL || ' - ' || FI.NMFILIAL AS LABELUNIDADE,
      'Todos' AS LABELLOJA,
      TO_CHAR(VD.DTENTRVENDA, 'DD/MM/YYYY') AS DATAVENDA,
      IC.CDFILIAL,
      FI.NMFILIAL,
      VE.CDVENDEDOR,
      VE.NMRAZSOCVEN,
      VE.CDVENDEDOR || ' - ' || VE.NMRAZSOCVEN AS GARCON,
      NVL(IC.CDSUPERVISOR, VD.CDSUPERVISOR) AS CDSUPERVISOR,
      OP.NMOPERADOR,
      VD.NRMESA,
      IC.CDPRODUTO,
      PR.CDARVPROD,
      PR.NMPRODUTO,
      IC.DSOBSCANITE,
      (
        IC.QTPRODVENDC * (IC.VRUNITVENDC + IC.VRUNITVENCLC)
      ) AS VRUNITVEND,
      VD.IDORIGEMVENDA,
      CASE
        WHEN NVL(IC.CDSUPERVISOR, VD.CDSUPERVISOR) is NULL THEN 'Nao atribuido'
        ELSE NVL(IC.CDSUPERVISOR, VD.CDSUPERVISOR) || ' - ' || OP.NMOPERADOR
      END SUPERVISOR,
      CASE
        IC.IDPRODPRODUZC
        WHEN 'S' THEN 'Produzido'
        ELSE 'Não Produzido'
      END IDPRODPRODUZC
    FROM VENDA VD
      JOIN ITVENDACAN IC ON VD.CDFILIAL = IC.CDFILIAL
      AND VD.NRSEQVENDA = IC.NRSEQVENDA
      AND VD.CDCAIXA = IC.CDCAIXA
      AND VD.NRORG = IC.NRORG
      JOIN FILIAL FI ON FI.CDFILIAL = IC.CDFILIAL
      AND FI.NRORG = IC.NRORG
      LEFT JOIN VENDEDOR VE ON VD.CDVENDEDOR = VE.CDVENDEDOR
      AND VD.NRORG = VE.NRORG
      JOIN PRODUTO PR ON IC.CDPRODUTO = PR.CDPRODUTO
      AND IC.NRORG = PR.NRORG
      LEFT JOIN OPERADOR OP ON OP.CDOPERADOR = NVL(IC.CDSUPERVISOR, VD.CDSUPERVISOR)
      AND IC.NRORG = OP.NRORG
      JOIN LOJA LO ON VD.CDFILIAL = LO.CDFILIAL
      AND VD.CDLOJA = LO.CDLOJA
      AND VD.NRORG = LO.NRORG
    WHERE (
        INSTR(:CDFILIAL, VD.CDFILIAL) <> 0
        OR (:TCDFILIAL = 'T')
      )
      AND (
        (INSTR(:CDLOJA, VD.CDFILIAL || VD.CDLOJA) <> 0)
        OR (:TCDLOJA = 'T')
      )
      AND (
        (INSTR(:CDVENDEDOR, VD.CDVENDEDOR) <> 0)
        OR (:TCDVENDEDOR = 'T')
      )
      AND VD.DTENTRVENDA BETWEEN TO_DATE(:DTINI, 'DD/MM/YYYY') AND TO_DATE(:DTFIN, 'DD/MM/YYYY')
      AND VD.IDSTATUSNFCE = 'R'
      AND (
        NVL(IC.IDPRODPRODUZC, 'N') IN (:IDPRODPRODUZC)
        OR (:TIDPRODPRODUZC = 'T')
      )
      AND VD.NRORG = :NRORG
      AND IC.DSOBSCANITE IS NOT NULL
      AND VD.NRSEQVENDA NOT IN (
        SELECT IVD.NRSEQVENDA
        FROM ITEMVENDA IVD
        WHERE IVD.CDFILIAL = :CDFILIAL
          AND IVD.CDCAIXA = VD.CDCAIXA
          AND IVD.NRSEQVENDA = VD.NRSEQVENDA
      )
    ORDER BY 1,4
  ";

  const GET_REL_CANC_ITEM_SINTETICO = "
      SELECT 'Todas' AS LABELFILIAL, 'Todas' AS LABELUNIDADE, 'Todos' AS LABELLOJA,
              TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY') AS DATAVENDA, 'Todas' AS CDFILIAL, 'Todas' AS NMFILIAL, 
              VE.CDVENDEDOR, VE.NMRAZSOCVEN,
              VE.CDVENDEDOR|| ' - ' ||VE.NMRAZSOCVEN AS GARCON,
              NVL(I.CDSUPERVISOR, V.CDOPERADOR) AS CDSUPERVISOR, O.NMOPERADOR, 
              V.NRMESA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.DSOBSCANITE, 
              (I.QTPRODVENDC * (I.VRUNITVENDC + I.VRUNITVENCLC)) AS VRUNITVEND,
              V.IDORIGEMVENDA, NVL(I.CDSUPERVISOR, V.CDOPERADOR)||' - '||O.NMOPERADOR AS SUPERVISOR,
              CASE I.IDPRODPRODUZC
               WHEN 'S'
               THEN 'Produzido'
               ELSE  'Não Produzido'
              END IDPRODPRODUZC    
        FROM VENDA V
        JOIN ITVENDACAN I
          ON V.CDFILIAL = I.CDFILIAL
         AND V.NRSEQVENDA = I.NRSEQVENDA
         AND V.CDCAIXA = I.CDCAIXA
         AND V.NRORG = I.NRORG
        JOIN FILIAL F
          ON F.CDFILIAL = I.CDFILIAL
         AND F.NRORG = I.NRORG
        LEFT JOIN VENDEDOR VE
          ON V.CDVENDEDOR = VE.CDVENDEDOR
         AND V.NRORG = VE.NRORG
        JOIN PRODUTO P
          ON I.CDPRODUTO = P.CDPRODUTO
         AND I.NRORG = P.NRORG
        JOIN OPERADOR O
          ON O.CDOPERADOR = NVL(I.CDSUPERVISOR, V.CDOPERADOR)
         AND I.NRORG = O.NRORG
        JOIN LOJA L
          ON V.CDFILIAL = L.CDFILIAL
         AND V.CDLOJA = L.CDLOJA
         AND V.NRORG = L.NRORG
       WHERE (INSTR(:CDFILIAL, V.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T' ))
         AND (( INSTR(:CDLOJA , V.CDFILIAL||V.CDLOJA) <> 0 ) OR (:TCDLOJA = 'T'))
         AND (( INSTR(:CDVENDEDOR , V.CDVENDEDOR) <> 0 ) OR (:TCDVENDEDOR = 'T'))
         AND V.DTENTRVENDA BETWEEN TO_DATE(:DTINI, 'DD/MM/YYYY') AND TO_DATE(:DTFIN, 'DD/MM/YYYY')
         AND (NVL(I.IDPRODPRODUZC, 'N') IN (:IDPRODPRODUZC) OR (:TIDPRODPRODUZC = 'T'))
      
      UNION ALL

      SELECT 'Todas' AS LABELFILIAL, 'Todas' AS LABELUNIDADE, 'Todos' AS LABELLOJA,
              TO_CHAR(VD.DTENTRVENDA, 'DD/MM/YYYY') AS DATAVENDA, 'Todas' AS CDFILIAL, 'Todas' AS NMFILIAL, 
              VE.CDVENDEDOR, VE.NMRAZSOCVEN,
              VE.CDVENDEDOR|| ' - ' ||VE.NMRAZSOCVEN AS GARCON,
              NVL(IC.CDSUPERVISOR, VD.CDOPERADOR) AS CDSUPERVISOR, OP.NMOPERADOR, 
              VD.NRMESA, IC.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, IC.DSOBSCANITE, 
              (IC.QTPRODVENDC * (IC.VRUNITVENDC + IC.VRUNITVENCLC)) AS VRUNITVEND,
              VD.IDORIGEMVENDA, NVL(IC.CDSUPERVISOR, VD.CDOPERADOR)||' - '||OP.NMOPERADOR AS SUPERVISOR,
              CASE IC.IDPRODPRODUZC
               WHEN 'S'
               THEN 'Produzido'
               ELSE  'Não Produzido'
              END IDPRODPRODUZC    
        FROM VENDA VD
        JOIN ITVENDACAN IC
          ON VD.CDFILIAL = IC.CDFILIAL
         AND VD.NRSEQVENDA = IC.NRSEQVENDA
         AND VD.CDCAIXA = IC.CDCAIXA
         AND VD.NRORG = IC.NRORG
        JOIN FILIAL FI
          ON FI.CDFILIAL = IC.CDFILIAL
         AND FI.NRORG = IC.NRORG
        LEFT JOIN VENDEDOR VE
          ON VD.CDVENDEDOR = VE.CDVENDEDOR
         AND VD.NRORG = VE.NRORG
        JOIN PRODUTO PR
          ON IC.CDPRODUTO = PR.CDPRODUTO
         AND IC.NRORG = PR.NRORG
        JOIN OPERADOR OP
          ON OP.CDOPERADOR = NVL(IC.CDSUPERVISOR, VD.CDOPERADOR)
         AND IC.NRORG = OP.NRORG
        JOIN LOJA LO
          ON VD.CDFILIAL = LO.CDFILIAL
         AND VD.CDLOJA = LO.CDLOJA
         AND VD.NRORG = LO.NRORG
       WHERE (INSTR(:CDFILIAL, VD.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T' ))
         AND (( INSTR(:CDLOJA , VD.CDFILIAL||VD.CDLOJA) <> 0 ) OR (:TCDLOJA = 'T'))
         AND (( INSTR(:CDVENDEDOR , VD.CDVENDEDOR) <> 0 ) OR (:TCDVENDEDOR = 'T'))
         AND VD.DTENTRVENDA BETWEEN TO_DATE(:DTINI, 'DD/MM/YYYY') AND TO_DATE(:DTFIN, 'DD/MM/YYYY')
         AND VD.IDSTATUSNFCE = 'R'
         AND IC.DSOBSCANITE IS NOT NULL
         AND (NVL(IC.IDPRODPRODUZC, 'N') IN (:IDPRODPRODUZC) OR (:TIDPRODPRODUZC = 'T'))
         AND VD.NRSEQVENDA NOT IN (SELECT IVD.NRSEQVENDA FROM ITEMVENDA IVD WHERE IVD.CDFILIAL = :CDFILIAL AND IVD.CDCAIXA = VD.CDCAIXA AND IVD.NRSEQVENDA = VD.NRSEQVENDA)
       ORDER BY 1, 4
  ";
}